/**
 * 
 */
package it.unicam.cs.tesei.graphs;

/**
 * Classe per gli archi di un grafo.
 * 
 * @author Luca Tesei
 *
 */
public class Edge<E, V> {
    private final E node1;

    private final E node2;

    private V label;

    private final boolean directed;

    private int color;

    /**
     * Costruisce un arco.
     * 
     * @param node1
     *            sorgente (solo in caso di arco diretto).
     * @param node2
     *            destinazione (solo in caso di arco diretto).
     * @param label
     *            etichetta.
     * @param directed
     *            true se l'arco è diretto, false altrimenti.
     */
    public Edge(E node1, E node2, V label, boolean directed) {
        this.node1 = node1;
        this.node2 = node2;
        this.label = label;
        this.color = Graph.COLOR_WHITE;
        this.directed = directed;
    }

    /**
     * Costruisce un arco.
     * 
     * @param node1
     *            sorgente (solo in caso di arco diretto).
     * @param node2
     *            destinazione (solo in caso di arco diretto).
     * @param label
     *            etichetta.
     * @param color
     *            il colore dell'arco.
     * @param directed
     *            true se l'arco è diretto, false altrimenti.
     */
    public Edge(E node1, E node2, V label, int color, boolean directed) {
        this.node1 = node1;
        this.node2 = node2;
        this.label = label;
        this.color = color;
        this.directed = directed;
    }

    /**
     * Restituisce il primo nodo di questo arco, la sorgente in caso di arco
     * diretto.
     * 
     * @return il primo nodo di questo arco, la sorgente in caso di arco
     *         diretto.
     */
    public E here() {
        return this.node1;
    }

    public E there() {
        return this.node2;
    }

    public void setLabel(V label) {
        this.label = label;

    }

    public V getLabel() {
        return this.label;
    }

    public void setColor(int color) {
        this.color = color;
    }

    public int getColor() {
        return this.color;
    }

    public boolean isDirected() {
        return this.directed;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + (directed ? 1231 : 1237);
        result = prime * result + ((node1 == null) ? 0 : node1.hashCode());
        result = prime * result + ((node2 == null) ? 0 : node2.hashCode());
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (!(obj instanceof Edge))
            return false;
        Edge other = (Edge) obj;
        if (directed != other.directed)
            return false;
        if (node1 == null) {
            if (other.node1 != null)
                return false;
        } else if (!node1.equals(other.node1))
            return false;
        if (node2 == null) {
            if (other.node2 != null)
                return false;
        } else if (!node2.equals(other.node2))
            return false;
        return true;
    }

}
